@@ -16,15 +16,12 @@ J.adapter.readers.xml.XmlReader.$XmlReader$NVPair$ ();
 Clazz.instantialize (this, arguments);
 }, J.adapter.readers.xml, "XmlReader", J.adapter.smarter.AtomSetCollectionReader);
 Clazz.prepareFields (c$, function () {
+this.parent = this;
 this.chars = JU.SB.newN (2000);
 this.domObj =  new Array (1);
 });
 Clazz.overrideMethod (c$, "initializeReader", 
 function () {
-this.initCML ();
-});
-Clazz.defineMethod (c$, "initCML", 
-function () {
 this.atts =  new java.util.Hashtable ();
 this.setMyError (this.parseXML ());
 this.continuing = false;
@@ -43,13 +40,16 @@ var saxReader = null;
 });
 Clazz.defineMethod (c$, "selectReaderAndGo", 
  function (saxReader) {
-this.asc =  new J.adapter.smarter.AtomSetCollection (this.readerName, this, null, null);
 var className = null;
 var pt = this.readerName.indexOf ("(");
 var name = (pt < 0 ? this.readerName : this.readerName.substring (0, pt));
 className = J.adapter.smarter.Resolver.getReaderClassBase (name);
+if (className.equals (this.getClass ().getName ())) {
+this.thisReader = this.parent = this;
+} else {
+this.asc =  new J.adapter.smarter.AtomSetCollection (this.readerName, this, null, null);
 if ((this.thisReader = this.getInterface (className)) == null) return "File reader was not found: " + className;
-try {
+}try {
 this.thisReader.processXml (this, saxReader);
 } catch (e) {
 if (Clazz.exceptionOf (e, Exception)) {
@@ -68,10 +68,11 @@ this.processXml2 (parent, saxReader);
 Clazz.defineMethod (c$, "processXml2", 
 function (parent, saxReader) {
 this.parent = parent;
+if (parent !== this) {
 this.asc = parent.asc;
 this.reader = parent.reader;
 this.atts = parent.atts;
-if (saxReader == null) {
+}if (saxReader == null) {
 this.attribs =  new Array (1);
 this.domObj =  new Array (1);
 var o = "";
@@ -91,7 +92,7 @@ this.createDomNodeJS ("xmlReader", null);
 }}, "J.adapter.readers.xml.XmlReader,~O");
 Clazz.defineMethod (c$, "createDomNodeJS", 
 function (id, data) {
-var applet = this.parent.vwr.html5Applet;
+var applet = (this.parent == null ? this : this.parent).vwr.html5Applet;
 var d = null;
 {
 if (!data)
@@ -123,7 +124,7 @@ d.innerHTML = data;
 Clazz.overrideMethod (c$, "applySymmetryAndSetTrajectory", 
 function () {
 try {
-if (this.parent == null) this.applySymTrajASCR ();
+if (this.parent == null || this.parent === this) this.applySymTrajASCR ();
  else this.parent.applySymmetryAndSetTrajectory ();
 } catch (e) {
 if (Clazz.exceptionOf (e, Exception)) {
@@ -204,7 +205,7 @@ function () {
 });
 Clazz.overrideMethod (c$, "finalizeSubclassReader", 
 function () {
-if (this.thisReader != null) this.thisReader.finalizeSubclassReader ();
+if (this.thisReader != null && this.thisReader !== this) this.thisReader.finalizeSubclassReader ();
 this.thisReader = null;
 });
 c$.$XmlReader$NVPair$ = function () {
