@@ -1,10 +1,11 @@
 Clazz.declarePackage ("J.adapter.writers");
-Clazz.load (["J.adapter.writers.CMLWriter", "java.util.Stack", "JU.SB"], "J.adapter.writers.CDXMLWriter", ["java.io.BufferedInputStream", "$.FileInputStream", "java.lang.NullPointerException", "JU.BinaryDocument", "$.PT"], function () {
+Clazz.load (["J.adapter.writers.CMLWriter", "java.util.Stack", "JU.SB"], "J.adapter.writers.CDXMLWriter", ["java.io.BufferedInputStream", "$.FileInputStream", "java.lang.NullPointerException", "java.util.HashMap", "$.Hashtable", "JU.BinaryDocument", "$.PT", "$.Rdr", "J.adapter.smarter.SmarterJmolAdapter"], function () {
 c$ = Clazz.decorateAsClass (function () {
 this.doc = null;
 this.objects = null;
 this.sb = null;
 this.sbpt = 0;
+this.id = 0;
 Clazz.instantialize (this, arguments);
 }, J.adapter.writers, "CDXMLWriter", J.adapter.writers.CMLWriter);
 Clazz.prepareFields (c$, function () {
@@ -27,13 +28,96 @@ c$.fromCDX = Clazz.defineMethod (c$, "fromCDX",
 function (binaryDoc) {
 return  new J.adapter.writers.CDXMLWriter ().cdxToCdxml (binaryDoc);
 }, "javajs.api.GenericBinaryDocument");
+c$.fromASC = Clazz.defineMethod (c$, "fromASC", 
+function (asc) {
+return  new J.adapter.writers.CDXMLWriter ().ascToCdxml (asc);
+}, "J.adapter.smarter.AtomSetCollection");
+Clazz.defineMethod (c$, "ascToCdxml", 
+ function (asc) {
+var a =  new J.adapter.smarter.SmarterJmolAdapter ();
+var atomIterator = a.getAtomIterator (asc);
+var bondIterator = a.getBondIterator (asc);
+J.adapter.writers.CMLWriter.openDocument (this.sb);
+J.adapter.writers.CDXMLWriter.appendHeader (this.sb);
+J.adapter.writers.CMLWriter.openTag (this.sb, "page");
+this.id = 0;
+this.openID (this.sb, "fragment");
+J.adapter.writers.CMLWriter.terminateTag (this.sb);
+var indexToID =  new java.util.Hashtable ();
+while (atomIterator.hasNext ()) {
+this.openID (this.sb, "n");
+indexToID.put (atomIterator.getUniqueID (), Integer.$valueOf (this.id));
+var xyz = atomIterator.getXYZ ();
+var ele = atomIterator.getElement ();
+var iso = atomIterator.getIsotope ();
+var charge = atomIterator.getFormalCharge ();
+J.adapter.writers.CMLWriter.addAttribute (this.sb, "p", J.adapter.writers.CDXMLWriter.cdxCoord (xyz.x, 200) + " " + J.adapter.writers.CDXMLWriter.cdxCoord (-xyz.y, 400));
+if (ele != 6) J.adapter.writers.CMLWriter.addAttribute (this.sb, "Element", "" + ele);
+if (iso != 0) J.adapter.writers.CMLWriter.addAttribute (this.sb, "Isotope", "" + iso);
+if (charge != 0) J.adapter.writers.CMLWriter.addAttribute (this.sb, "Charge", "" + charge);
+J.adapter.writers.CMLWriter.terminateEmptyTag (this.sb);
+}
+while (bondIterator.hasNext ()) {
+this.openID (this.sb, "b");
+var id1 = bondIterator.getAtomUniqueID1 ();
+var id2 = bondIterator.getAtomUniqueID2 ();
+var order = "1";
+var display = null;
+var display2 = null;
+var bo = bondIterator.getEncodedOrder ();
+switch (bo) {
+case 515:
+order = "1.5";
+display2 = "Dash";
+break;
+case 514:
+order = "2";
+break;
+default:
+if ((bo & 0x07) != 0) order = "" + (bo & 0x07);
+break;
+}
+switch (bo) {
+case 1025:
+order = "1";
+display = "WedgeBegin";
+break;
+case 1041:
+order = "1";
+display = "WedgedHashBegin";
+break;
+case 1057:
+order = "1";
+display = "Wavy";
+break;
+}
+J.adapter.writers.CMLWriter.addAttribute (this.sb, "B", indexToID.get (id1).toString ());
+J.adapter.writers.CMLWriter.addAttribute (this.sb, "E", indexToID.get (id2).toString ());
+if (!order.equals ("1")) J.adapter.writers.CMLWriter.addAttribute (this.sb, "Order", order);
+if (display != null) J.adapter.writers.CMLWriter.addAttribute (this.sb, "Display", display);
+if (display2 != null) J.adapter.writers.CMLWriter.addAttribute (this.sb, "Display2", display2);
+J.adapter.writers.CMLWriter.terminateEmptyTag (this.sb);
+}
+J.adapter.writers.CMLWriter.closeTag (this.sb, "fragment");
+J.adapter.writers.CMLWriter.closeTag (this.sb, "page");
+J.adapter.writers.CMLWriter.closeTag (this.sb, "CDXML");
+return this.sb.toString ();
+}, "J.adapter.smarter.AtomSetCollection");
+c$.cdxCoord = Clazz.defineMethod (c$, "cdxCoord", 
+ function (x, off) {
+return "" + (off + Math.round (x / 1.45 * 14.4 * 100) / 100);
+}, "~N,~N");
+Clazz.defineMethod (c$, "openID", 
+ function (sb, name) {
+J.adapter.writers.CMLWriter.startOpenTag (sb, name);
+J.adapter.writers.CMLWriter.addAttribute (sb, "id", "" + ++this.id);
+}, "JU.SB,~S");
 Clazz.defineMethod (c$, "cdxToCdxml", 
  function (doc) {
 this.doc = doc;
 try {
 J.adapter.writers.CMLWriter.openDocument (this.sb);
-this.sb.append ("<!DOCTYPE CDXML SYSTEM \"http://www.cambridgesoft.com/xml/cdxml.dtd\" >\n");
-this.sb.append ("<CDXML BondLength=\"14.40\">\n");
+J.adapter.writers.CDXMLWriter.appendHeader (this.sb);
 doc.setStreamData (null, false);
 doc.seek (22);
 this.processObject (doc.readShort ());
@@ -49,6 +133,13 @@ throw e;
 }
 return this.sb.toString ();
 }, "javajs.api.GenericBinaryDocument");
+c$.appendHeader = Clazz.defineMethod (c$, "appendHeader", 
+ function (sb) {
+sb.append ("<!DOCTYPE CDXML SYSTEM \"http://www.cambridgesoft.com/xml/cdxml.dtd\" >\n");
+J.adapter.writers.CMLWriter.startOpenTag (sb, "CDXML");
+J.adapter.writers.CMLWriter.addAttributes (sb, J.adapter.writers.CDXMLWriter.cdxAttributes);
+J.adapter.writers.CMLWriter.terminateTag (sb);
+}, "JU.SB");
 Clazz.defineMethod (c$, "processObject", 
  function (type) {
 var id = this.doc.readInt ();
@@ -396,7 +487,28 @@ throw e;
 }
 }
 }, "~A");
+c$.fromString = Clazz.defineMethod (c$, "fromString", 
+function (vwr, type, mol) {
+var htParams =  new java.util.HashMap ();
+htParams.put ("filter", "filetype=" + type);
+var o;
+try {
+o = vwr.getModelAdapter ().getAtomSetCollectionFromReader ("", JU.Rdr.getBR (mol), htParams);
+} catch (e) {
+if (Clazz.exceptionOf (e, Exception)) {
+e.printStackTrace ();
+return null;
+} else {
+throw e;
+}
+}
+if (Clazz.instanceOf (o, String)) {
+return null;
+}return J.adapter.writers.CDXMLWriter.fromASC (o);
+}, "JV.Viewer,~S,~S");
 Clazz.defineStatics (c$,
+"STANDARD_BOND_LENGTH", 14.4,
+"cdxAttributes",  Clazz.newArray (-1, ["HashSpacing", "2.50", "MarginWidth", "1.60", "LineWidth", "0.60", "BoldWidth", "2", "BondLength", "14.40", "BondSpacing", "18"]),
 "kCDXProp_ChemicalWarning", 0x0010,
 "kCDXProp_2DPosition", 0x0200,
 "kCDXProp_Node_Type", 0x0400,
