@@ -26,7 +26,7 @@ function () {
 });
 Clazz.defineMethod (c$, "getChiralityForAtoms", 
 function (data) {
-if (data.bsAtoms.cardinality () == 0) return;
+if (data.bsAtoms.isEmpty ()) return;
 this.data = data;
 this.doTrack = data.isTracker ();
 this.ptIDLogger = 0;
