@@ -318,7 +318,7 @@ rpn.addOp (JS.T.tokenRightParen);
 case 1086326786:
 case 1086326785:
 case 1073742328:
-case 1678381065:
+case 1812599299:
 case 1086326788:
 case 1073742329:
 case 1111490587:
@@ -569,7 +569,7 @@ case 2097194:
 rpn.addXBs (JU.BSUtil.copy (this.vwr.slm.getHiddenSet ()));
 break;
 case 12293:
-rpn.addXBs (this.vwr.getMotionFixedAtoms ());
+rpn.addXBs (this.vwr.getMotionFixedAtoms (-1));
 break;
 case 2097192:
 rpn.addXBs (JU.BSUtil.copyInvert (this.vwr.slm.getHiddenSet (), ac));
@@ -686,7 +686,7 @@ case 2:
 rpn.addXNum (instruction);
 break;
 case 10:
-var bs1 = JU.BSUtil.copy (value);
+var bs1 = (Clazz.instanceOf (value, JM.BondSet) ? value : JU.BSUtil.copy (value));
 rpn.addXBs (bs1);
 break;
 case 8:
@@ -1148,7 +1148,7 @@ switch (tok) {
 case 1140850689:
 case 1677721602:
 if (this.chk) return bs;
-bsNew = (tok == 1140850689 ? (isAtoms ? bs : this.vwr.ms.getAtoms (1677721602, bs)) : (isAtoms ? JM.BondSet.newBS (this.vwr.getBondsForSelectedAtoms (bs), null) : bs));
+bsNew = (tok == 1140850689 ? (isAtoms ? bs : this.vwr.ms.getAtoms (1677721602, bs)) : (isAtoms ? JM.BondSet.newBS (this.vwr.getBondsForSelectedAtoms (bs)) : bs));
 var i;
 switch (minmaxtype) {
 case 32:
@@ -1170,6 +1170,7 @@ return bsNew;
 case 1086324745:
 switch (minmaxtype) {
 case 0:
+case 32:
 case 1073742327:
 return this.getCmdExt ().getBitsetIdent (bs, null, tokenValue, useAtomMap, index, isExplicitlyAll);
 }
