@@ -13,12 +13,11 @@ function (pattern, isSmarts, isEmpirical) {
 this.clearExceptions ();
 var search = JS.SmilesParser.newSearch ("/nostereo/" + pattern, isSmarts, true);
 search.createTopoMap (null);
-search.nodes = search.targetAtoms;
+search.nodes = search.target.nodes;
 return search.getMolecularFormula (!isSmarts, null, isEmpirical);
 }, "~S,~B,~B");
 Clazz.defineMethod (c$, "clearExceptions", 
  function () {
-this.okMF = true;
 JS.InvalidSmilesException.clear ();
 });
 Clazz.overrideMethod (c$, "getSmiles", 
@@ -50,53 +49,18 @@ var search = JS.SmilesParser.newSearch (target, false, true);
 var array = this.matchPriv (pattern, null, 0, null, null, false, flags, 3, search);
 for (var i = array.length; --i >= 0; ) {
 var a = array[i];
-for (var j = a.length; --j >= 0; ) a[j] = (search.targetAtoms[a[j]]).mapIndex;
+for (var j = a.length; --j >= 0; ) a[j] = (search.target.nodes[a[j]]).mapIndex;
 
 }
 return array;
 }, "~S,~S,~N");
-Clazz.overrideMethod (c$, "hasStructure", 
-function (pattern, smilesSet, flags) {
-var ret =  Clazz.newIntArray (smilesSet.length, 0);
-if ((flags & 1) != 1) {
-flags = flags | 2;
-}this.clearExceptions ();
-pattern = JS.SmilesParser.cleanPattern (pattern);
-try {
-var search = JS.SmilesParser.newSearch (pattern, true, false);
-for (var i = 0; i < smilesSet.length; i++) {
-var smiles = JS.SmilesParser.cleanPattern (smilesSet[i]);
-var searchTarget = JS.SmilesParser.newSearch (smiles, false, true);
-searchTarget.setFlags (searchTarget.flags | JS.SmilesParser.getFlags (pattern));
-try {
-ret[i] = (this.matchPattern (search, null, 0, null, null, false, flags | 8, 5, searchTarget) === Boolean.TRUE ? 1 : 0);
-} catch (e) {
-if (Clazz.exceptionOf (e, Exception)) {
-ret[i] = -1;
-e.printStackTrace ();
-} else {
-throw e;
-}
-}
-}
-} catch (e) {
-if (Clazz.exceptionOf (e, Exception)) {
-if (JU.Logger.debugging) e.printStackTrace ();
-if (JS.InvalidSmilesException.getLastError () == null) this.clearExceptions ();
-throw  new JS.InvalidSmilesException (JS.InvalidSmilesException.getLastError ());
-} else {
-throw e;
-}
-}
-return ret;
-}, "~S,~A,~N");
 Clazz.overrideMethod (c$, "getAtoms", 
 function (target) {
 this.clearExceptions ();
 target = JS.SmilesParser.cleanPattern (target);
 var search = JS.SmilesParser.newSearch (target, false, true);
 search.createTopoMap ( new JU.BS ());
-return search.targetAtoms;
+return search.target.nodes;
 }, "~S");
 Clazz.overrideMethod (c$, "getRelationship", 
 function (smiles1, smiles2) {
@@ -127,9 +91,10 @@ smiles = JU.PT.rep (smiles, "!@@", "@");
 return smiles;
 }, "~S");
 Clazz.overrideMethod (c$, "getSubstructureSet", 
-function (pattern, atoms, ac, bsSelected, flags) {
-return this.matchPriv (pattern, atoms, ac, bsSelected, null, true, flags | JS.SmilesParser.getFlags (pattern), 1, null);
-}, "~S,~A,~N,JU.BS,~N");
+function (pattern, target, ac, bsSelected, flags) {
+var atoms = (Clazz.instanceOf (target, JS.SmilesSearch) ? null : target);
+return this.matchPriv (pattern, atoms, ac, bsSelected, null, true, flags | JS.SmilesParser.getFlags (pattern.toString ()), 1, (atoms == null ? target : null));
+}, "~O,~O,~N,JU.BS,~N");
 Clazz.overrideMethod (c$, "getMMFF94AtomTypes", 
 function (smarts, atoms, ac, bsSelected, ret, vRings) {
 this.clearExceptions ();
@@ -138,9 +103,7 @@ var search = null;
 var flags = (770);
 search = sp.parse ("");
 search.exitFirstMatch = false;
-search.targetAtoms = atoms;
-search.targetAtomCount = Math.abs (ac);
-search.setSelected (bsSelected);
+search.target.setAtoms (atoms, Math.abs (ac), bsSelected);
 search.flags = flags;
 search.getRingData (vRings, true, true);
 search.asVector = false;
@@ -217,9 +180,11 @@ Clazz.defineMethod (c$, "matchPriv",
  function (pattern, atoms, ac, bsSelected, bsAromatic, doTestAromatic, flags, mode, searchTarget) {
 this.clearExceptions ();
 try {
+var isCompiled = (Clazz.instanceOf (pattern, JS.SmilesSearch));
+if (isCompiled) flags |= 2;
 var isSmarts = ((flags & 2) == 2);
-var search = JS.SmilesParser.newSearch (pattern, isSmarts, false);
-if (searchTarget != null) searchTarget.setFlags (searchTarget.flags | JS.SmilesParser.getFlags (pattern));
+var search = (isCompiled ? pattern : JS.SmilesParser.newSearch (pattern == null ? null : pattern.toString (), isSmarts, false));
+if (searchTarget != null) searchTarget.setFlags (searchTarget.flags | JS.SmilesParser.getFlags (pattern.toString ()));
 return this.matchPattern (search, atoms, ac, bsSelected, bsAromatic, doTestAromatic, flags, mode, searchTarget);
 } catch (e) {
 if (Clazz.exceptionOf (e, Exception)) {
@@ -230,22 +195,25 @@ throw  new JS.InvalidSmilesException (JS.InvalidSmilesException.getLastError ())
 throw e;
 }
 }
-}, "~S,~A,~N,JU.BS,JU.BS,~B,~N,~N,JS.SmilesSearch");
+}, "~O,~A,~N,JU.BS,JU.BS,~B,~N,~N,JS.SmilesSearch");
 Clazz.defineMethod (c$, "matchPattern", 
  function (search, atoms, ac, bsSelected, bsAromatic, doTestAromatic, flags, mode, searchTarget) {
 var isSmarts = ((flags & 2) == 2);
-var isTopo = ((flags & 16384) == 16384);
+this.okMF = true;
 if (searchTarget != null) {
+if (searchTarget.targetSet) {
+search.setTarget (searchTarget);
+} else {
 search.haveSmilesTarget = true;
 bsAromatic =  new JU.BS ();
 searchTarget.createTopoMap (bsAromatic);
-atoms = searchTarget.targetAtoms;
-ac = searchTarget.targetAtoms.length;
+atoms = searchTarget.target.nodes;
+ac = searchTarget.target.nodes.length;
 if (isSmarts) {
 var a1 = searchTarget.elementCounts;
 var a2 = search.elementCounts;
-var n = searchTarget.elementNumberMax;
-if (n == search.elementNumberMax) {
+var n = search.elementNumberMax;
+if (n <= searchTarget.elementNumberMax) {
 for (var i = 1; i <= n; i++) {
 if (a1[i] < a2[i]) {
 this.okMF = false;
@@ -265,19 +233,17 @@ break;
 }}
 } else {
 this.okMF = false;
-}}}if (this.okMF) {
+}}}}if (this.okMF) {
 if (!isSmarts && !search.patternAromatic) {
 if (bsAromatic == null) bsAromatic =  new JU.BS ();
 search.normalizeAromaticity (bsAromatic);
 search.isNormalized = true;
-}search.targetAtoms = atoms;
-search.targetAtomCount = ac;
-search.setSelected (bsSelected);
-if (ac != 0 && (bsSelected == null || !bsSelected.isEmpty ())) {
-var is3D = !(Clazz.instanceOf (atoms[0], JS.SmilesAtom));
+}if (!search.targetSet) search.target.setAtoms (atoms, ac, bsSelected);
+if (search.targetSet || ac != 0 && (bsSelected == null || !bsSelected.isEmpty ())) {
+var is3D = search.targetSet || !(Clazz.instanceOf (atoms[0], JS.SmilesAtom));
 search.getSelections ();
-if (!doTestAromatic) search.bsAromatic = bsAromatic;
-search.setRingData (null, null, is3D || doTestAromatic || search.patternAromatic);
+if (!doTestAromatic) search.target.bsAromatic = bsAromatic;
+if (!search.target.hasRingData (flags)) search.setRingData (null, null, is3D || doTestAromatic || search.patternAromatic);
 search.exitFirstMatch = ((flags & 8) == 8);
 search.mapUnique = ((flags & 128) == 128);
 }}switch (mode) {
@@ -388,6 +354,54 @@ throw e;
 }
 }
 }, "~S");
+Clazz.overrideMethod (c$, "compileSmartsPattern", 
+function (pattern) {
+pattern = JS.SmilesParser.cleanPattern (pattern);
+return JS.SmilesParser.newSearch (pattern, true, false);
+}, "~S");
+Clazz.overrideMethod (c$, "compileSearchTarget", 
+function (atoms, atomCount, bsSelected) {
+var ss =  new JS.SmilesSearch ();
+ss.target.setAtoms (atoms, atomCount, bsSelected);
+ss.targetSet = true;
+return ss;
+}, "~A,~N,JU.BS");
+Clazz.overrideMethod (c$, "hasStructure", 
+function (pattern, smilesSet, flags) {
+var ret =  Clazz.newIntArray (smilesSet.length, 0);
+if ((flags & 1) != 1) {
+flags = flags | 2;
+}this.clearExceptions ();
+pattern = JS.SmilesParser.cleanPattern (pattern);
+try {
+var search = JS.SmilesParser.newSearch (pattern, true, false);
+for (var i = 0; i < smilesSet.length; i++) {
+var smiles = JS.SmilesParser.cleanPattern (smilesSet[i]);
+var searchTarget = JS.SmilesParser.newSearch (smiles, false, true);
+searchTarget.setFlags (searchTarget.flags | JS.SmilesParser.getFlags (pattern));
+try {
+this.clearExceptions ();
+ret[i] = (this.matchPattern (search, null, 0, null, null, false, flags | 8, 5, searchTarget) === Boolean.TRUE ? 1 : 0);
+} catch (e) {
+if (Clazz.exceptionOf (e, Exception)) {
+ret[i] = -1;
+e.printStackTrace ();
+} else {
+throw e;
+}
+}
+}
+} catch (e) {
+if (Clazz.exceptionOf (e, Exception)) {
+if (JU.Logger.debugging) e.printStackTrace ();
+if (JS.InvalidSmilesException.getLastError () == null) this.clearExceptions ();
+throw  new JS.InvalidSmilesException (JS.InvalidSmilesException.getLastError ());
+} else {
+throw e;
+}
+}
+return ret;
+}, "~S,~A,~N");
 Clazz.defineStatics (c$,
 "MODE_BITSET", 0x01,
 "MODE_ARRAY", 0x02,
