@@ -68,14 +68,8 @@ for (var i = this.pixels.length; --i >= 0; ) this.pixels[i] = this.pixels[i] & -
 if (params.containsKey ("captureRootExt") || !params.containsKey ("captureMode")) return;
 this.interlaced = false;
 this.capturing = true;
-try {
-this.byteCount = (params.get ("captureByteCount")).intValue ();
-} catch (e) {
-if (Clazz.exceptionOf (e, Exception)) {
-} else {
-throw e;
-}
-}
+var c = params.get ("captureByteCount");
+if (c != null) this.byteCount = c.intValue ();
 switch ("maec".indexOf ((params.get ("captureMode")).substring (0, 1))) {
 case 0:
 params.put ("captureMode", "add");
