@@ -5,6 +5,7 @@
 
 // see JSmolApi.js for public user-interface. All these are private functions
 
+// BH 2023.01.19 Jmol._allowKeyboardFocus = true; can be set to false if page jumping is a problem.
 // BH 8/25/2022 fixes getFileData response "" from empty file returning "OK" instead of ""
 // BH 8/15/2022 adds .lut for binary
 // BH 6/23/2022 implements Jmol._lastAppletID via setMouseOwner
@@ -191,6 +192,7 @@ Jmol = (function(document) {
 				before calling Jmol.getApplet(), limits for applet size can be overriden.
 				2048 standard for GeoWall (http://geowall.geo.lsa.umich.edu/home.html)
 		*/
+		_allowKeyboardFocus: true, // set false on page if mouse moves and clicks cause too much scrolling of window
 		_appletCssClass: "",
 		_appletCssText: "",
 		_fileCache: null, // enabled by Jmol.setFileCaching(applet, true/false)
@@ -1747,7 +1749,8 @@ Jmol = (function(document) {
 		Jmol.$bind(canvas, 'mousedown touchstart', function(ev) {
       if (doIgnore(ev))
         return true;
-      canvas.focus();
+      if (Jmol._allowKeyboardFocus)	
+    	  canvas.focus();
 			Jmol._setMouseOwner(canvas, true);
 			ev.stopPropagation();
       var ui = ev.target["data-UI"];
@@ -1786,7 +1789,9 @@ Jmol = (function(document) {
 		Jmol.$bind(canvas, 'mousemove touchmove', function(ev) { // touchmove
       if (doIgnore(ev))
         return true;
-      canvas.focus();
+      if (Jmol._allowKeyboardFocus)	
+    	  canvas.focus(); //in Chrome and Edge, this forces a jump 
+      // if the Jmol window is not within viewing bounds
 		  // defer to console or menu when dragging within this canvas
 			if (Jmol._mouseOwner && Jmol._mouseOwner != canvas && Jmol._mouseOwner.isDragging) {
         if (!Jmol._mouseOwner.mouseMove)
